@@ -1,5 +1,5 @@
 /* gif.h - Interface to the LCDF GIF library.
-   Copyright (C) 1997-2010 Eddie Kohler, kohler@cs.ucla.edu
+   Copyright (C) 1997-2011 Eddie Kohler, kohler@cs.ucla.edu
    This file is part of the LCDF GIF library.
 
    The GIF library is free software. It is distributed under the GNU General
@@ -129,7 +129,10 @@ int		Gif_ImageNumber(Gif_Stream *, Gif_Image *);
 #define		Gif_ImageUserData(gfi)		((gfi)->userdata)
 #define		Gif_SetImageUserData(gfi, v)	((gfi)->userdata = v)
 
-typedef		void (*Gif_ReadErrorHandler)(const char *, int, void *);
+typedef		void (*Gif_ReadErrorHandler)(int is_error,
+					     const char *error_text,
+					     int frame_number,
+					     void *user_data);
 
 #define		Gif_UncompressImage(gfi)     Gif_FullUncompressImage((gfi),0,0)
 int		Gif_FullUncompressImage(Gif_Image*,Gif_ReadErrorHandler,void*);
@@ -232,6 +235,8 @@ struct Gif_Record {
 #define GIF_READ_CONST_RECORD		4
 #define GIF_READ_TRAILING_GARBAGE_OK	8
 #define GIF_WRITE_CAREFUL_MIN_CODE_SIZE	1
+#define GIF_WRITE_EAGER_CLEAR		2
+#define GIF_WRITE_OPTIMIZE		4
 
 Gif_Stream *	Gif_ReadFile(FILE *);
 Gif_Stream *	Gif_FullReadFile(FILE *, int flags, Gif_ReadErrorHandler,
