@@ -1,5 +1,5 @@
 /* support.c - Support functions for gifsicle.
-   Copyright (C) 1997-2008 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2011 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -23,18 +23,37 @@ int no_warnings = 0;
 
 
 static void
-verror(int seriousness, const char *fmt, va_list val)
+verror(int need_file, int seriousness, const char *fmt, va_list val)
 {
   char pattern[BUFSIZ];
   char buffer[BUFSIZ];
-  verbose_endline();
+  static char *printed_file = 0;
+  static int just_printed_context = 0;
+  const char *iname = input_name ? input_name : "<stdin>", *prefix = "";
+
+  if (printed_file && strcmp(printed_file, iname) != 0) {
+      free(printed_file);
+      printed_file = 0;
+  }
+  if (need_file && !printed_file) {
+      if (mode != BLANK_MODE && mode != MERGING && nested_mode != MERGING) {
+	  fprintf(stderr, "%s: While processing '%s':\n", program_name, iname);
+	  just_printed_context = 1;
+	  prefix = "  ";
+      }
+      printed_file = malloc(strlen(iname) + 1);
+      strcpy(printed_file, iname);
+  } else if (just_printed_context && seriousness == 0)
+      prefix = "  ";
+  else
+      just_printed_context = 0;
 
   if (seriousness > 2)
-    sprintf(pattern, "%s: fatal error: %%s\n", program_name);
+      sprintf(pattern, "%s:%s fatal error: %%s\n", program_name, prefix);
   else if (seriousness == 1)
-    sprintf(pattern, "%s: warning: %%s\n", program_name);
+      sprintf(pattern, "%s:%s warning: %%s\n", program_name, prefix);
   else
-    sprintf(pattern, "%s: %%s\n", program_name);
+      sprintf(pattern, "%s:%s %%s\n", program_name, prefix);
 
   if (seriousness > 1)
     error_count++;
@@ -44,6 +63,7 @@ verror(int seriousness, const char *fmt, va_list val)
   /* try and keep error messages together (no interleaving of error messages
      from two gifsicle processes in the same command line) by calling fprintf
      only once */
+  verbose_endline();
   if (strlen(fmt) + strlen(pattern) < BUFSIZ) {
     sprintf(buffer, pattern, fmt);
     vfprintf(stderr, buffer, val);
@@ -60,35 +80,35 @@ fatal_error(const char *message, ...)
 {
   va_list val;
   va_start(val, message);
-  verror(3, message, val);
+  verror(0, 3, message, val);
   va_end(val);
   exit(EXIT_USER_ERR);
 }
 
 void
-error(const char *message, ...)
+error(int need_file, const char *message, ...)
 {
   va_list val;
   va_start(val, message);
-  verror(2, message, val);
+  verror(need_file, 2, message, val);
   va_end(val);
 }
 
 void
-warning(const char *message, ...)
+warning(int need_file, const char *message, ...)
 {
   va_list val;
   va_start(val, message);
-  verror(1, message, val);
+  verror(need_file, 1, message, val);
   va_end(val);
 }
 
 void
-warncontext(const char *message, ...)
+warncontext(int need_file, const char *message, ...)
 {
   va_list val;
   va_start(val, message);
-  verror(0, message, val);
+  verror(need_file, 0, message, val);
   va_end(val);
 }
 
@@ -176,7 +196,7 @@ Animation options: Also --no-OPTION and --same-OPTION.\n\
   -d, --delay TIME              Set frame delay to TIME (in 1/100sec).\n\
   -D, --disposal METHOD         Set frame disposal to METHOD.\n\
   -l, --loopcount[=N]           Set loop extension to N (default forever).\n\
-  -O, --optimize[=LEV]          Optimize output GIFs.\n\
+  -O, --optimize[=LEVEL]        Optimize output GIFs.\n\
   -U, --unoptimize              Unoptimize input GIFs.\n\
 \n");
   printf("\
@@ -254,7 +274,8 @@ safe_puts(const char *s, uint32_t len, FILE *f)
   for (; len > 0; len--, s++)
     if (*s < ' ' || *s >= 0x7F || *s == '\\') {
       if (last_safe != s) {
-	if (fwrite(last_safe, 1, s - last_safe, f) != s - last_safe)
+	size_t n = s - last_safe;
+	if (fwrite(last_safe, 1, n, f) != n)
 	  return;
       }
       last_safe = s + 1;
@@ -272,7 +293,8 @@ safe_puts(const char *s, uint32_t len, FILE *f)
       }
     }
   if (last_safe != s) {
-    if (fwrite(last_safe, 1, s - last_safe, f) != s - last_safe)
+    size_t n = s - last_safe;
+    if (fwrite(last_safe, 1, n, f) != n)
       return;
   }
 }
@@ -831,7 +853,7 @@ read_text_colormap(FILE *f, const char *name)
       if (green > 255) green = 255;
       if (blue > 255) blue = 255;
       if (ncol >= 256) {
-	error("%s: maximum 256 colors allowed in colormap", name);
+	error(0, "%s: maximum 256 colors allowed in colormap", name);
 	break;
       } else {
 	col[ncol].red = red;
@@ -850,7 +872,7 @@ read_text_colormap(FILE *f, const char *name)
   }
 
   if (ncol == 0) {
-    error("'%s' doesn't seem to contain a colormap", name);
+    error(0, "%s: file not in colormap format", name);
     Gif_DeleteColormap(cm);
     return 0;
   } else {
@@ -875,7 +897,7 @@ read_colormap_file(const char *name, FILE *f)
     else
       f = fopen(name, "rb");
     if (!f) {
-      error("%s: %s", name, strerror(errno));
+      error(0, "%s: %s", name, strerror(errno));
       return 0;
     }
   }
@@ -888,12 +910,12 @@ read_colormap_file(const char *name, FILE *f)
   if (c == 'G') {
     Gif_Stream *gfs = Gif_ReadFile(f);
     if (!gfs)
-      error("'%s' doesn't seem to contain a GIF", name);
+      error(0, "%s: file not in GIF format", name);
     else if (!gfs->global)
-      error("can't use '%s' as a palette (no global color table)", name);
+      error(0, "%s: can't use as palette (no global color table)", name);
     else {
       if (gfs->errors)
-	warning("there were errors reading '%s'", name);
+	warning(0, "%s: there were errors reading this GIF", name);
       cm = Gif_CopyColormap(gfs->global);
     }
 
@@ -1052,14 +1074,14 @@ find_color_or_error(Gif_Color *color, Gif_Stream *gfs, Gif_Image *gfi,
       return color->pixel;
     else {
       if (color_context)
-	  error("%s color out of range", color_context);
+	  error(0, "%s color out of range", color_context);
       return -1;
     }
   }
 
   index = Gif_FindColor(gfcm, color);
   if (index < 0 && color_context)
-    error("%s color not in colormap", color_context);
+    error(0, "%s color not in colormap", color_context);
   return index;
 }
 
@@ -1081,10 +1103,10 @@ set_background(Gif_Stream *gfs, Gt_OutputData *output_data)
     if (output_data->background.haspixel) {
 	if (gfs->images[0]->transparent >= 0) {
 	    static int context = 0;
-	    warning("irrelevant background color");
+	    warning(1, "irrelevant background color");
 	    if (!context) {
-		warncontext("(The background will appear transparent because");
-		warncontext("the first image contains transparency.)");
+		warncontext(1, "(The background will appear transparent because");
+		warncontext(1, "the first image contains transparency.)");
 		context = 1;
 	    }
 	}
@@ -1125,9 +1147,9 @@ set_background(Gif_Stream *gfs, Gt_OutputData *output_data)
     /* Report conflicts. */
     if (conflict || (want_transparent && gfs->images[0]->transparent < 0)) {
 	static int context = 0;
-	warning("input images have conflicting background colors");
+	warning(1, "input images have conflicting background colors");
 	if (!context) {
-	  warncontext("(This means some animation frames may appear incorrect.)");
+	  warncontext(1, "(This means some animation frames may appear incorrect.)");
 	  context = 1;
 	}
     }
@@ -1156,16 +1178,18 @@ fix_total_crop(Gif_Stream *dest, Gif_Image *srci, int merger_index)
   assert(dest->nimages > 0);
   prev_image = dest->images[dest->nimages - 1];
   if (merger_index < nmerger - 1)
-      next_fr = merger[merger_index + 1];
+    next_fr = merger[merger_index + 1];
 
   /* Don't save identifiers since the frame that was to be identified, is
      gone. Save comments though. */
   if (!fr->no_comments && srci->comment && next_fr) {
-    if (!next_fr->comment) next_fr->comment = Gif_NewComment();
+    if (!next_fr->comment)
+      next_fr->comment = Gif_NewComment();
     merge_comments(next_fr->comment, srci->comment);
   }
   if (fr->comment && next_fr) {
-    if (!next_fr->comment) next_fr->comment = Gif_NewComment();
+    if (!next_fr->comment)
+      next_fr->comment = Gif_NewComment();
     merge_comments(next_fr->comment, fr->comment);
     Gif_DeleteComment(fr->comment);
     fr->comment = 0;
@@ -1221,11 +1245,10 @@ handle_flip_and_screen(Gif_Stream *dest, Gif_Image *desti, Gt_Frame *fr)
 }
 
 static void
-analyze_crop(int nmerger, Gt_Crop *crop)
+analyze_crop(int nmerger, Gt_Crop *crop, int compress_immediately)
 {
-  int i;
+  int i, nframes = 0;
   int l = 0x7FFFFFFF, r = 0, t = 0x7FFFFFFF, b = 0;
-  int nframes = 0;
 
   /* count frames to which this crop applies */
   for (i = 0; i < nmerger; i++)
@@ -1236,20 +1259,31 @@ analyze_crop(int nmerger, Gt_Crop *crop)
   for (i = 0; i < nmerger; i++)
     if (merger[i]->crop == crop) {
       Gt_Frame *fr = merger[i];
-      int ll = (nframes > 1 ? 0 : fr->image->left);
-      int tt = (nframes > 1 ? 0 : fr->image->top);
-      int ww = (nframes > 1 ? fr->stream->screen_width : fr->image->width);
-      int hh = (nframes > 1 ? fr->stream->screen_height : fr->image->height);
+      int ll, tt, rr, bb;
+      if (nframes <= 1) {
+	ll = fr->image->left;
+	tt = fr->image->top;
+	rr = fr->image->left + fr->image->width;
+	bb = fr->image->top + fr->image->height;
+      } else {
+	ll = tt = 0;
+	rr = fr->stream->screen_width;
+	bb = fr->stream->screen_height;
+      }
       if (ll < l)
 	l = ll;
       if (tt < t)
 	t = tt;
-      if (ll + ww > r)
-	r = ll + ww;
-      if (tt + hh > b)
-	b = tt + hh;
+      if (rr > r)
+	r = rr;
+      if (bb > b)
+	b = bb;
     }
 
+  if (t > b)
+    /* total crop */
+    l = r = t = b = 0;
+
   crop->x = crop->spec_x + l;
   crop->y = crop->spec_y + t;
   crop->w = crop->spec_w <= 0 ? (r - crop->x) + crop->spec_w : crop->spec_w;
@@ -1258,10 +1292,99 @@ analyze_crop(int nmerger, Gt_Crop *crop)
   crop->top_offset = crop->y;
   if (crop->x < 0 || crop->y < 0 || crop->w <= 0 || crop->h <= 0
       || crop->x + crop->w > r || crop->y + crop->h > b) {
-    error("cropping dimensions don't fit image");
+    error(1, "cropping dimensions don't fit image");
     crop->ready = 2;
   } else
     crop->ready = 1;
+
+  /* Remove transparent edges. */
+  if (crop->transparent_edges && crop->ready == 1) {
+    int have_l = crop->x, have_t = crop->y,
+      have_r = crop->x + crop->w, have_b = crop->y + crop->h;
+    l = t = 0x7FFFFFFF, r = b = 0;
+
+    for (i = 0;
+	 i < nmerger && (l > have_l || t > have_t || r < have_r || b < have_b);
+	 ++i)
+      if (merger[i]->crop == crop) {
+	Gt_Frame *fr = merger[i];
+	Gif_Image *srci = fr->image;
+	int ll = constrain(have_l, srci->left, have_r),
+	  tt = constrain(have_t, srci->top, have_b),
+	  rr = constrain(have_l, srci->left + srci->width, have_r),
+	  bb = constrain(have_t, srci->top + srci->height, have_b);
+
+	if (srci->transparent >= 0) {
+	  int x, y;
+	  uint8_t **img;
+	  Gif_UncompressImage(srci);
+	  img = srci->img;
+
+	  /* Move top edge down over transparency */
+	  while (tt < bb && tt < t) {
+	    uint8_t *data = img[tt - srci->top];
+	    for (x = 0; x < srci->width; ++x)
+	      if (data[x] != srci->transparent)
+		goto found_top;
+	    ++tt;
+	  }
+
+	found_top:
+	  /* Move bottom edge up over transparency */
+	  while (bb > tt + 1 && bb > b) {
+	    uint8_t *data = img[bb - 1 - srci->top];
+	    for (x = 0; x < srci->width; ++x)
+	      if (data[x] != srci->transparent)
+		goto found_bottom;
+	    --bb;
+	  }
+
+	found_bottom:
+	  if (tt < bb) {
+	    /* Move left edge right over transparency */
+	    while (ll < rr && ll < l) {
+	      for (y = tt - srci->top; y < bb - srci->top; ++y)
+		if (img[y][ll - srci->left] != srci->transparent)
+		  goto found_left;
+	      ++ll;
+	    }
+
+	  found_left:
+	    /* Move right edge left over transparency */
+	    while (rr > ll + 1 && rr > r) {
+	      for (y = tt - srci->top; y < bb - srci->top; ++y)
+		if (img[y][rr - 1 - srci->left] != srci->transparent)
+		  goto found_right;
+	      --rr;
+	    }
+	  }
+
+	found_right:
+	  if (compress_immediately)
+	    Gif_ReleaseUncompressedImage(srci);
+	}
+
+	if (tt < bb) {
+	  if (ll < l)
+	    l = ll;
+	  if (tt < t)
+	    t = tt;
+	  if (rr > r)
+	    r = rr;
+	  if (bb > b)
+	    b = bb;
+	}
+      }
+
+    if (t > b)
+      crop->w = crop->h = 0;
+    else {
+      crop->x = l;
+      crop->y = t;
+      crop->w = r - l;
+      crop->h = b - t;
+    }
+  }
 }
 
 
@@ -1283,7 +1406,7 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
   nmerger = 0;
   merger_flatten(fset, f1, f2);
   if (nmerger == 0) {
-    error("empty output GIF not written");
+    error(1, "empty output GIF not written");
     return 0;
   }
 
@@ -1294,7 +1417,7 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
       s += ((merger[i]->image->width * merger[i]->image->height) / 1024) + 1;
     *huge_stream = (s > 200 * 1024); /* 200 MB */
     if (*huge_stream && !compress_immediately) {
-      warning("huge GIF, conserving memory (processing may take a while)");
+      warning(1, "huge GIF, conserving memory (processing may take a while)");
       compress_immediately = 1;
     }
   }
@@ -1333,7 +1456,7 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
       merger[i]->crop->ready = all_same_compressed_ok = 0;
   for (i = 0; i < nmerger; i++)
     if (merger[i]->crop && !merger[i]->crop->ready)
-      analyze_crop(nmerger, merger[i]->crop);
+      analyze_crop(nmerger, merger[i]->crop, compress_immediately);
 
   /* copy stream-wide information from output_data */
   if (output_data->loopcount > -2)
@@ -1364,13 +1487,23 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
 	Gif_AddExtension(dest, gfex, i);
 	gfex = next;
       }
+      fr->extensions = 0;
     }
 
     /* Make a copy of the image and crop it if we're cropping */
     if (fr->crop) {
+      int preserve_total_crop;
       srci = Gif_CopyImage(fr->image);
       Gif_UncompressImage(srci);
-      if (!crop_image(srci, fr->crop, dest->nimages == 0)) {
+
+      /* Zero-delay frames are a special case.  You might think it was OK to
+	 get rid of totally-cropped delay-0 frames, but many browsers treat
+	 zero-delay frames as 100ms.  So don't completely crop a zero-delay
+	 frame: leave it around.  Problem reported by Calle Kabo. */
+      preserve_total_crop = (dest->nimages == 0 || fr->delay == 0
+			     || (fr->delay < 0 && srci->delay == 0));
+
+      if (!crop_image(srci, fr->crop, preserve_total_crop)) {
 	/* We cropped the image out of existence! Be careful not to make 0x0
 	   frames. */
 	fix_total_crop(dest, srci, i);
